# plot_utils.R
#

# Functions for green/red background colors vector
myRGBColVec <- function (nrgcols = 12) {
  k <- trunc(nrgcols/2)
  
  if (2 * k == nrgcols) {
    r <- c(rev(seq(0, 1, length = k)), rep(0, k))
    g <- c(rep(0, k), seq(0, 1, length = k))
    #change
    #colvec <- rgb(r, g, rep(0, 2 * k))
    colvec <- rgb(g, r, rep(0, 2 * k))
  } else {
    r <- c(rev(seq(1/(2 * k - 1), 1, length = k)), rep(0, k + 1))
    g <- c(rep(0, k + 1), seq(1/(2 * k - 1), 1, length = k))
    #colvec <- rgb(r, g, rep(0, 2 * k + 1))
    colvec <- rgb(g, r, rep(0, 2 * k + 1))
  }
  
  colvec
}

#for blue/yellow background colors vector
blueYellowBColVec <- function (nrgcols = 12) {
  k <- trunc(nrgcols/2)
  if (2 * k == nrgcols) {
    r <- c(rep(0, k), seq(0, 1, length = k))
    g <- r
    b <- c(rev(seq(0, 1, length = k)), rep(0, k))
    #change
    #colvec <- rgb(r, g, b, rep(0, 2 * k))
    colvec <- rgb(r, g, b)
  } else {
    r <- c(rep(0, k + 1), seq(1/(2 * k - 1), 1, length = k))
    g <- r
    b <- c(rev(seq(1/(2 * k - 1), 1, length = k)), rep(0, k + 1))
    #colvec <- rgb(r, g, b, rep(0, 2 * k + 1))
    colvec <- rgb(r, g, b)
    
  }

  colvec
}

# heatmap top color labels
changeColorGroups <- function(inputColors) {
  ncolors <- inputColors
  
  for (k in 1:length(ncolors)) {
    if (identical(ncolors[k], "red")) { 
      ncolors[k] <- "black" #blue"
    } else if (identical(ncolors[k], "blue")) {
      ncolors[k] <- "grey" #yellow"
    }
  }
  
  ncolors
}

whiteColumn2IngoreLast <- function(inputColors) {
  ncolors <- inputColors
  rows <- length(ncolors[, 2]) - 1
  
  for (k in 1:rows) {
    ncolors[k, 2] <- "white"
    ncolors[k, 1] <- "white"
  }
  
  ncolors
}

#Heatmap image my.row, n.col, my.col2
heatmap1XPlotToFile <- function(y, rowValue, nCol, colColor, outfileName, pctitle, time_stamp) {
  heatmap_file <- paste(paste(outfileName, "heatmap", sep="_"), 'png', sep = ".");
  #png(filename=heatmap_file, width=1040, height=800, family = "Courier")
  #CairoPNG(filename=heatmap_file, width=1040, height=720, pointsize = 12, family = "Courier")
  png(filename=heatmap_file, width=11, height=8.5, unit='in', res=72, type="cairo")
  plotHeatmap(y, rowValue, nCol, colColor, pctitle, time_stamp)
  dev.off()
}

# 2X Heatmap image my.row, n.col, my.col2
heatmapPlotToFile <- function(y, rowValue, nCol, colColor, outfileName, pctitle, time_stamp) {
  heatmap_file <- paste(paste(outfileName, "heatmap", sep="_"), 'png', sep = ".");
  #png(filename=heatmap_file, width=1040, height=800, family = "Courier")
  #CairoPNG(filename=heatmap_file, width=1040, height=720, pointsize = 12, family = "Courier")
  res2X <- 144
  png(filename=heatmap_file, width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  plotHeatmap(y, rowValue, nCol, colColor, pctitle, time_stamp)
  dev.off()
  
  heatmap_file
}

plotHeatmap <- function (y, rowValue, nCol, colColor, pctitle, time_stamp) {
  #color_maps <- myRGBColVec(nCol)
  color_maps <- blueYellowBColVec(nCol)
  col_ch_Colors <- colColor
  col_ch_Colors <- changeColorGroups(colColor)
  #col_ch_Colors <- whiteIngoreLast(col_ch_Colors)
  col_ch_Colors <- whiteColumn2IngoreLast(col_ch_Colors)
  
  class_label <- "NDDI"
  #class_label
  if (col_ch_Colors[31, 2] == "black") {
    class_label <- 'DDI'
  }
  
  col_ch_Colors[31, 2] <- "white"
                 
  hc <- heatmap3(y, Rowv = rowValue, Colv = NA, scale = "row", margins = c(15, 10), #main = pctitle,
                 cexRow=0.65, cexCol=1, col = color_maps, ColSideColors = col_ch_Colors)

  if (!is.null(pctitle)) {
    mtext(pctitle, side=3, line=-3, at=c(0.45), cex=1)
  }
  
  mtext(c("DNA Damage Inducing (DDI)", "Non DNA Damage Inducing (NDDI)"), side=3, line=-4.5, at=c(0.24, 0.46), adj=0, cex=0.9)
  mtext(time_stamp, side=1, line=3, cex=0.8)
  mtext(class_label, side=3, line=-5, at=c(0.74), adj=0, cex=0.7)
  
  #left-v
  segments(0.229, 0.84, 0.229, 0.86, col= 'black', lwd=1)
  #mid-v
  segments(0.452, 0.84, 0.452, 0.86, col= 'black', lwd=1)
  #right-v
  segments(0.727, 0.84, 0.727, 0.86, col= 'black', lwd=1)
  #horizental
  segments(0.23, 0.85, 0.726, 0.85, col= 'black', lwd=1)
}

# Cluster image 2X
clusterPlotToFile <- function(x, outfileName, pctitle, time_stamp) {
  cluster_file <- paste(paste(outfileName, "dendro+pca", sep="_"), 'png', sep = ".");
  #png(filename=cluster_file, width=1040, height=800)
  res2X <- 144
  #CairoPNG(filename=cluster_file, width=1040, height=720, pointsize = 12, family = "Courier")
  png(filename=cluster_file, width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  plotCluster(x, pctitle, time_stamp)
  dev.off()
  
  cluster_file
}

# Cluster image
clusterPlotToFile0 <- function(x, outfileName, pctitle, time_stamp) {
  cluster_file <- paste(paste(outfileName, "dendro+pca", sep="_"), 'png', sep = ".");
  #png(filename=cluster_file, width=1040, height=800)
  #CairoPNG(filename=cluster_file, width=1040, height=720, pointsize = 12, family = "Courier")
  png(filename=cluster_file, width=11, height=8.5, unit='in', res=72, type="cairo")
  plotCluster(x, pctitle, time_stamp)
  dev.off()
}

plotCluster <- function (x, pctitle, time_stamp) {
  my.text <- colnames(x)
  # row.loc <- apply(x[,1:29],1,mean)
  # row.scale <- apply(x[,1:29],1,sd)
  # x <- (x - row.loc)/row.scale
  
  my.pca <- prcomp(t(x[,1:29]), retx = TRUE, center = FALSE, scale. = FALSE)
  pc1 <- as.vector(my.pca$rotation[,1]) %*% x
  pc2 <- as.vector(my.pca$rotation[,2]) %*% x
  
  #par(mfrow = c(1,2), mar = c(5,4,4,2))
  par(mar = c(5,4,4,2), oma=c(1,1,2,1), ps=12)
  layout(matrix(c(1,2), byrow=T, ncol=2))
  
  small_cex <- 0.5
  large_cex <- small_cex * 3
  plot(pc1, pc2, type = "n", main = "", xlab = "PC 1", ylab = "PC 2")
  points(pc1[1:13], pc2[1:13], pch = 20, cex=small_cex, col = "red")# pch = 16
  points(pc1[14:29], pc2[14:29], pch = 20, cex=small_cex, col = "blue")
  points(pc1[30:ncol(x)], pc2[30:ncol(x)], pch = 17, cex=large_cex, col = "dark green")
  
  text(pc1[1:13], pc2[1:13], labels = my.text[1:13], col = "red", cex = 0.7, pos = 1)
  text(pc1[14:29], pc2[14:29], labels = my.text[14:29], col = "blue", cex = 0.7, pos = 1)
  text(pc1[30:ncol(x)], pc2[30:ncol(x)], labels = my.text[30:ncol(x)], col = "dark green", cex = 0.7, pos = 1)
  
  #mtext(time_stamp, side=1, line=3, adj=1, cex=0.7)
  
  abline(v = 0, col = "red", lwd = 2)
  
  chem.dist <- dist(t(x))
  
  dendro <- hclust(chem.dist, method = "average")
  dendro <- as.dendrogram(dendro)
  
  # Assigning the labels of dendrogram object with new colors:
  labels_colors(dendro) <- c(rep("red", 13), rep("blue", 16), rep("dark green", 5))[order.dendrogram(dendro)]
  
  par(mar = c(3,1,1,10), cex = 0.7)
  plot(dendro, horiz=T)
  
  if (!is.null(pctitle)) {
    mtext(pctitle, side=3, line=-1, outer=T, adj=0.5, cex=1.5)
  }

  mtext(time_stamp, side=1, outer=T, line=-0.5, adj=0.49, cex=0.8)
  mtext("Euclidean distance", side=1, outer=T, line=-4.5, at=(0.7), cex=0.8)
  
  chem.dist
}

# 2X Heatmap image my.row, n.col, my.col2
heatmap_plot_png <- function(y, rowValue, nCol, colColor, outfileName, pctitle, time_stamp, file_dir) {
  heatmap_file <- paste(paste(outfileName, "heatmap", sep="_"), 'png', sep = ".");
  #png(filename=heatmap_file, width=1040, height=800, family = "Courier")
  #CairoPNG(filename=heatmap_file, width=1040, height=720, pointsize = 12, family = "Courier")
  res2X <- 144
  png(filename=file.path(file_dir, heatmap_file), width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  plotHeatmap(y, rowValue, nCol, colColor, pctitle, time_stamp)
  dev.off()
  
  heatmap_file
}

# Cluster image 2X
cluster_plot_png <- function(x, outfileName, pctitle, time_stamp, file_dir) {
  cluster_file <- paste(paste(outfileName, "dendro+pca", sep="_"), 'png', sep = ".");
  #png(filename=cluster_file, width=1040, height=800)
  res2X <- 144
  #CairoPNG(filename=cluster_file, width=1040, height=720, pointsize = 12, family = "Courier")
  png(filename=file.path(file_dir, cluster_file), width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  plotCluster(x, pctitle, time_stamp)
  dev.off()
  
  cluster_file
}

# Plot PCA only, DDI(1, 13, 29, 39), HDACi(1, 10, 20, 21), tested with DDI
plot_pca <- function(x, index_1, pctitle, time_stamp, colors = c("red", "blud", "dark green")) {
  index_max <- ncol(x)
  index_2 <- index_1 + 1
  index_3 <- index_max - 1
  my.text <- colnames(x)
  my.pca <- prcomp(t(x[,1:index_3]), retx = TRUE, center = FALSE, scale. = FALSE)
  pc1 <- as.vector(my.pca$rotation[,1]) %*% x
  pc2 <- as.vector(my.pca$rotation[,2]) %*% x
  
  #par(mfrow = c(1,2), mar = c(5,4,4,2))
  par(mar = c(5,4,4,2), oma=c(1,1,2,1), ps=12)
  # layout(matrix(c(1,2), byrow=T, ncol=2))
  
  small_cex <- 0.5
  large_cex <- small_cex * 3
  plot(pc1, pc2, type = "n", main = pctitle, xlab = "PC 1", ylab = "PC 2")
  points(pc1[1:index_1], pc2[1:index_1], pch = 20, cex=small_cex, col = "red")# pch = 16
  points(pc1[index_2:index_3], pc2[index_2:index_3], pch = 20, cex=small_cex, col = "blue")
  points(pc1[index_max:ncol(x)], pc2[index_max:ncol(x)], pch = 17, cex=large_cex, col = "dark green")
  
  text(pc1[1:index_1], pc2[1:index_1], labels = my.text[1:index_1], col = "red", cex = 0.7, pos = 1)
  text(pc1[index_2:index_3], pc2[index_2:index_3], labels = my.text[index_2:index_3], col = "blue", cex = 0.7, pos = 1)
  text(pc1[index_max:ncol(x)], pc2[index_max:ncol(x)], labels = my.text[index_max:ncol(x)], col = "dark green", cex = 0.7, pos = 1)
  
  #mtext(time_stamp, side=1, line=3, adj=1, cex=0.7)
  
  abline(v = 0, col = "red", lwd = 2)
}

# Plot cluster only, tested with DDI
plot_cluster <- function(x, pctitle, time_stamp, counts = c(13, 16, 1), colors = c("red", "blud", "dark green")) {
  chem.dist <- dist(t(x))
  dendro <- hclust(chem.dist, method = "average")
  dendro <- as.dendrogram(dendro)
  
  # Assigning the labels of dendrogram object with new colors:
  # labels_colors(dendro) <- c(rep("red", 13), rep("blue", 16), rep("dark green", 5))[order.dendrogram(dendro)]
  labels_colors(dendro) <- c(rep("red", counts[1]), rep("blue", counts[2]), rep("dark green", counts[3]))[order.dendrogram(dendro)]
  
  par(mar = c(4,2,2,10), cex = 0.7)
  plot(dendro, horiz=T, main=pctitle)
  mtext(time_stamp, side=1, outer=T, line=-1.5, adj=0.49, cex=0.8)
  mtext("Euclidean distance", side=1, outer=T, line=-5.2, cex=0.8)
}

# For HDACi heatmap
plot_hdaci_heatmap <- function (y, rowValue, nCol, colColor, pctitle, time_stamp) {
  #color_maps <- myRGBColVec(nCol)
  color_maps <- blueYellowBColVec(nCol)
  col_ch_Colors <- colColor
  col_ch_Colors <- changeColorGroups(colColor)
  #col_ch_Colors <- whiteIngoreLast(col_ch_Colors)
  col_ch_Colors <- whiteColumn2IngoreLast(col_ch_Colors)
  
  class_label <- "NHDACI"
  row_max <- nrow(col_ch_Colors)
  #class_label
  if (col_ch_Colors[row_max, 2] == "black") {
    class_label <- 'HDACI'
  }
  
  col_ch_Colors[row_max, 2] <- "white"
  
  # hc <- heatmap3(y, Rowv = rowValue, Colv = NA, scale = "row", margins = c(15, 10),  main = pctitle,
  hc <- heatmap3(y, Rowv = rowValue, Colv = NA, scale = "row", margins = c(15, 10), #main = pctitle,
                 cexRow=0.65, cexCol=1, col = color_maps, ColSideColors = col_ch_Colors)
  
  if (!is.null(pctitle)) {
    mtext(pctitle, side=3, line=-3, at=c(0.45), cex=1)
  }
  
  mtext(c("HDACI", "Non-HDACI)"), side=3, line=-4.5, at=c(0.32, 0.54), adj=0, cex=0.9)
  mtext(time_stamp, side=1, line=3, cex=0.8)
  mtext(class_label, side=3, line=-5, at=c(0.73), adj=0, cex=0.7)
  
  #left-v
  segments(0.229, 0.84, 0.229, 0.86, col= 'black', lwd=1)
  #mid-v
  segments(0.47, 0.84, 0.47, 0.86, col= 'black', lwd=1)
  #right-v
  segments(0.715, 0.84, 0.715, 0.86, col= 'black', lwd=1)
  #horizental
  segments(0.23, 0.85, 0.714, 0.85, col= 'black', lwd=1)
}

pdf_hdaci_heatmap <- function(out_path, file_out_prefix, file_suffix, y, my.row, n.col, my.col2, title_plot, date_stamp) {
  file_heatmap <- file.path(out_path, paste(file_out_prefix, file_suffix, sep = "_"))
  CairoPDF(file = file_heatmap, width = 11, height = 8.5, family = "Courier")
  
  # blue-yellow colors for heat map to replace default green-red colors
  plot_hdaci_heatmap(y, my.row, n.col, my.col2, title_plot, date_stamp)
  
  dev.off()
}

png_hdaci_heatmap <- function(out_path, file_out_prefix, file_suffix, y, my.row, n.col, my.col2, title_plot, date_stamp) {
  res2X <- 144
  file_heatmap <- file.path(out_path, paste(file_out_prefix, file_suffix, sep = "_"))
  png(filename = file_heatmap, width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")

  # blue-yellow colors for heat map to replace default green-red colors
  plot_hdaci_heatmap(y, my.row, n.col, my.col2, title_plot, date_stamp)
  dev.off()
}

plot_hdaci_chem_dendro <- function(dendro, title_plot) {
  my.col <- rep("dark green", length(dendro$labels))
  flag <- 1:10
  my.col[flag] <- "red"
  flag <- 11:20
  my.col[flag] <- "blue"
  
  # Assigning the labels of dendrogram object with new colors:
  dendrogram <- as.dendrogram(dendro)
  labels_colors(dendrogram) <- my.col[order.dendrogram(dendrogram)]
  
  par(mar = c(3,1,1,10), cex = 1)
  # plot(dendrogram, horiz=T, main = paste(name_label, "Euclidean, Average Linkage (log2)", sep = ": "))
  plot(dendrogram, horiz=T, main = title_plot)
  # mtext(date_stamp, side=1, line=3, cex=0.8)
}

pdf_hdaci_chem_dendro <- function(out_path, file_out_prefix, file_suffix, dendro, title_plot) {
  file_dendro <- file.path(out_path, paste(file_out_prefix, file_suffix, sep = "_"))
  CairoPDF(file = file_dendro, width = 11, height = 8.5, family = "Courier")
  plot_hdaci_chem_dendro(dendro, title_plot)
  dev.off()
}

png_hdaci_chem_dendro <- function(out_path, file_out_prefix, file_suffix, dendro, title_plot) {
  file_dendro <- file.path(out_path, paste(file_out_prefix, file_suffix, sep = "_"))
  res2X <- 144
  png(filename = file_dendro, width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  plot_hdaci_chem_dendro(dendro, title_plot)
  dev.off()
}

plot_hdaci_chem_pca <- function(x_scaled, z, title_plot) {
  my.text <- colnames(x_scaled)
  
  pc1 <- as.vector(z$pcl.1) %*% x_scaled
  pc2 <- as.vector(z$pcl.2) %*% x_scaled
  
  plot(pc1, pc2, type = "n", main = title_plot, xlab = "PC 1", ylab = "PC 2")
  
  flag <- 1:10
  points(pc1[flag], pc2[flag], pch = 16, col = "red")
  text(pc1[flag], pc2[flag], labels = my.text[flag], col = "red", cex = 0.5, pos = 1)
  
  flag <- 11:20
  points(pc1[flag], pc2[flag], pch = 16, col = "blue")
  text(pc1[flag], pc2[flag], labels = my.text[flag], col = "blue", cex = 0.5, pos = 1)
  
  flag <- 21:ncol(x_scaled)
  points(pc1[flag], pc2[flag], pch = 16, col = "dark green")
  text(pc1[flag], pc2[flag], labels = my.text[flag], col = "dark green", cex = 0.5, pos = 1)
  
  abline(v = 0, col = "red", lwd = 2)
}

pdf_hdaci_chem_pca <- function(out_path, file_out_prefix, file_suffix, x_scaled, z, title_plot) {
  file_pca <- file.path(out_path, paste(file_out_prefix, file_suffix, sep = "_"))
  CairoPDF(file = file_pca, width = 11, height = 8.5, family = "Courier")
  plot_hdaci_chem_pca(x_scaled, z, title_plot)
  dev.off()
}

png_hdaci_chem_pca <- function(out_path, file_out_prefix, file_suffix, x_scaled, z, title_plot) {
  file_pca <- file.path(out_path, paste(file_out_prefix, file_suffix, sep = "_"))
  res2X <- 144
  png(filename = file_pca, width=11 * res2X, height=8.5 * res2X, res=res2X, type="cairo")
  plot_hdaci_chem_pca(x_scaled, z, title_plot)
  dev.off()
}
